@@ -1,17 +1,17 @@
 package org.gcube.applicationsupportlayer.social.mailing;
 
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.IOException;
+import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Date;
-import java.util.Properties;
 
-import javax.mail.Address;
 import javax.mail.Message;
+import javax.mail.MessagingException;
+import javax.mail.Multipart;
 import javax.mail.Session;
-import javax.mail.Transport;
 import javax.mail.internet.InternetAddress;
+import javax.mail.internet.MimeBodyPart;
 import javax.mail.internet.MimeMessage;
+import javax.mail.internet.MimeMultipart;
 
 import org.gcube.portal.custom.communitymanager.OrganizationsUtil;
 import org.gcube.portal.databook.shared.Notification;
@@ -31,6 +31,21 @@ import com.liferay.portal.util.PortalUtil;
 public class EmailPlugin {
 
 	private static final Logger _log = LoggerFactory.getLogger(EmailPlugin.class);
+	protected static final int SECONDS2WAIT = 60;
+
+	private static EmailPlugin singleton;
+	
+	public static EmailPlugin getInstance() {
+		if (singleton == null)
+			singleton = new EmailPlugin();
+		return singleton;
+	}
+	
+	private EmailPlugin() {		
+		new EmailNotificationsConsumer().start();
+	}
+	
+	protected static ArrayList<NotificationMail> BUFFER_EMAILS = new ArrayList<NotificationMail>();
 	
 	private static String getHTMLEmail(Notification notification2Save, String userFirstName, String portalURL, String email) {
 
@@ -41,7 +56,7 @@ public class EmailPlugin {
 		String sender = notification2Save.getSenderFullName();
 		if (notification2Save.getType() == NotificationType.DOCUMENT_WORKFLOW_STEP_REQUEST_TASK)
 			sender = "";
-		
+
 		String portalHost = portalURL.replaceAll("https://", "");
 		portalHost = portalHost.replaceAll("http://", "");
 
@@ -68,7 +83,37 @@ public class EmailPlugin {
 		.append("If you have received this communication in error, please notify the <sender> and destroy and delete any copies you may have received.")
 		.append("</div></p>")
 		.append("</body>");
-	
+
+		return body.toString();
+
+	}
+
+	private static String getTextEmail(Notification notification2Save, String userFirstName, String portalURL, String email) {
+
+		String removedMarkup = notification2Save.getDescription().replaceAll("&amp;", "&");
+		removedMarkup = removedMarkup.replaceAll("&gt;", ">");
+		removedMarkup = removedMarkup.replaceAll("&lt;", "<");
+
+		String sender = notification2Save.getSenderFullName();
+		if (notification2Save.getType() == NotificationType.DOCUMENT_WORKFLOW_STEP_REQUEST_TASK)
+			sender = "";
+
+		String portalHost = portalURL.replaceAll("https://", "");
+		portalHost = portalHost.replaceAll("http://", "");
+
+		StringBuilder body = new StringBuilder();
+
+		body.append("Dear ").append(userFirstName).append(",")  //dear <user>
+		.append("\n").append(sender).append(" ").append(removedMarkup) // has done something
+		.append("\nsee: ").append(portalURL).append(notification2Save.getUri())
+		.append("\n\n\n----\n")
+		.append("This message was sent to ")
+		.append(email)
+		.append(" by ")
+		.append(portalHost)
+		.append(" If you don't want to receive these emails in the future, please unsubscribe here: ")
+		.append(portalURL).append("/group/data-e-infrastructure-gateway/notifications .");
+
 		return body.toString();
 
 	}
@@ -77,8 +122,26 @@ public class EmailPlugin {
 		StringBuilder actionLink = new StringBuilder("<a style=\"color:#3B5998; text-decoration:none\" target=\"_blank\" href=\"");
 		return completeActonLinkByNotificationType(notification2Save, actionLink, portalURL);
 	}
-
-	public static void sendNotification(Notification notification2Save, String vreName, String portalName, String senderEmail) {
+	
+	
+	/**
+	 * enqueue the message to send
+	 * @param notification2Save
+	 * @param vreName
+	 * @param portalName
+	 * @param senderEmail
+	 */
+	public void sendNotification(Notification notification2Save,	String vreName,	String portalName, String senderEmail)  {
+		EmailNotificationProducer thread = new EmailNotificationProducer(new NotificationMail(notification2Save, vreName, portalName, senderEmail));
+		thread.start();
+	}
+	
+	protected static Message getMessageNotification(
+			Session session, 
+			Notification notification2Save, 
+			String vreName, 
+			String portalName, 
+			String senderEmail) throws Exception {
 
 		UserModel user = null;
 		String portalUrl = null;
@@ -87,36 +150,38 @@ public class EmailPlugin {
 			portalUrl = PortalUtil.getPortalURL(OrganizationsUtil.getCompany().getVirtualHost(), PortalUtil.getPortalPort(), true);
 		} catch (Exception e1) {
 			e1.printStackTrace();
+			_log.warn("While trying to get email for user/group: " + notification2Save.getUserid());
+			return null;
 		} 
 		String email = user.getEmailAddress();
 
-		Properties props = System.getProperties();
-		String mailServiceHost = "localhost";
-		props.put("mail.smtp.host", mailServiceHost);
-		String mailServicePort = "25";
-		props.put("mail.smtp.port", mailServicePort);
-		Session session = Session.getDefaultInstance(props, null);
-		session.setDebug(true);
-		Message mimeMessage = new MimeMessage(session);
+		Message msg2Return = new MimeMessage(session);
 
-		try {
-			// EMAIL SENDER
-			Address from = new InternetAddress(senderEmail, portalName);
-		    mimeMessage.setHeader("Content-Type", "text/html; charset=UTF-8");
-			mimeMessage.setFrom(from);
-			Address address = new InternetAddress(email);
-			mimeMessage.addRecipient(Message.RecipientType.TO, address);
 
-			mimeMessage.setSubject(getSubjectByNotificationType(notification2Save, portalUrl, vreName, user.getFirstName()));
-			mimeMessage.setContent(getHTMLEmail(notification2Save, user.getFirstName(), portalUrl, email), "text/html; charset=UTF-8");
-			mimeMessage.setSentDate(new Date());
-			Transport.send(mimeMessage);
-			_log.trace("notification email sent successfully");
-		} catch (Exception e) {
-			e.printStackTrace();
-			_log.error("While sending the notification email:", e);
-		}
+		// EMAIL SENDER
+		msg2Return.setHeader("Content-Type", "text/html; charset=UTF-8");
+		msg2Return.setFrom(new InternetAddress(senderEmail, portalName));
+		msg2Return.addRecipient(Message.RecipientType.TO, new InternetAddress(email));
+
+		msg2Return.setSubject(getSubjectByNotificationType(notification2Save, portalUrl, vreName, user.getFirstName()));
+
+		final MimeBodyPart textPart = new MimeBodyPart();
+		textPart.setContent(getTextEmail(notification2Save, user.getFirstName(), portalUrl, email), "text/plain"); 
+
+		final MimeBodyPart htmlPart = new MimeBodyPart();
+		htmlPart.setContent(getHTMLEmail(notification2Save, user.getFirstName(), portalUrl, email), "text/html; charset=UTF-8");
+
+		final Multipart mp = new MimeMultipart("alternative");
+		mp.addBodyPart(textPart);
+		mp.addBodyPart(htmlPart);
+		// Set Multipart as the message's content
+		msg2Return.setContent(mp);
+
+		msg2Return.setSentDate(new Date());
+
+		return msg2Return;
 	}
+
 	/**
 	 * construct the subjec of the email
 	 * @param notification2Save
@@ -137,13 +202,15 @@ public class EmailPlugin {
 		case WP_FOLDER_REMOVEDUSER:
 			return "Removed user in a shared folder";
 		case WP_FOLDER_SHARE:
-			return "Folder sharing notification";
+			return notification2Save.getSenderFullName()+ " shared a folder with you";
 		case WP_ITEM_NEW:
-			return "New item in a shared folder";
+			return notification2Save.getSenderFullName()+ " added an item in a shared folder";
 		case WP_ITEM_DELETE:
-			return "Deleted item in a shared folder";
+			return notification2Save.getSenderFullName()+ " deleted an item in a shared folder";
 		case WP_ITEM_UPDATED:
-			return "Updated item in a shared folder";
+			return notification2Save.getSenderFullName()+ " updated an item in a shared folder";
+		case WP_ITEM_RENAMED:
+			return notification2Save.getSenderFullName()+ " renamed an item in a shared folder";
 		case OWN_COMMENT:
 			return notification2Save.getSenderFullName() + " commented on your post";
 		case MENTION:
@@ -189,9 +256,9 @@ public class EmailPlugin {
 	 * @return
 	 */
 	private static String completeActonLinkByNotificationType(Notification notification2Save, StringBuilder actionLink,  String portalURL) {
-		
+
 		actionLink.append(portalURL).append(notification2Save.getUri());  
-		
+
 		switch (notification2Save.getType()) {
 		case LIKE:		
 			actionLink.append("\">.").append(" See the Post").append("</a>");
@@ -272,10 +339,10 @@ public class EmailPlugin {
 			actionLink.append("\">.").append("").append("</a>");
 			break;			
 		}
-		
+
 		return actionLink.toString();
 	}
-		
+
 }
 
 
